/*********************************************************************
 *
 *                  RTLL Demo
 *
 *********************************************************************
 * FileName:        main.c
 * Dependencies:
 * Processor:       PIC32
 *
 * Complier:        MPLAB C32
 *                  MPLAB IDE
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 * Microchip RTLL  PIC32 Software.
 * Copyright  2008 Microchip Technology Inc.  All rights reserved.
 * 
 * Microchip licenses the Software for your use with Microchip microcontrollers
 * and Microchip digital signal controllers pursuant to the terms of the
 * Non-Exclusive Software License Agreement accompanying this Software.
 *
 * SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY
 * OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION,
 * ANY WARRANTY OF MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS
 * FOR A PARTICULAR PURPOSE.
 * MICROCHIP AND ITS LICENSORS ASSUME NO RESPONSIBILITY FOR THE ACCURACY,
 * RELIABILITY OR APPLICATION OF THE SOFTWARE AND DOCUMENTATION.
 * IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED
 * UNDER CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH
 * OF WARRANTY, OR OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT
 * DAMAGES OR EXPENSES INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL,
 * SPECIAL, INDIRECT, PUNITIVE OR CONSEQUENTIAL DAMAGES, LOST PROFITS
 * OR LOST DATA, COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY,
 * SERVICES, OR ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT LIMITED
 * TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
 *
 ********************************************************************/

// configuration bit settings, Fcy=80MHz, Fpb=80MHz
#pragma config POSCMOD=XT, FNOSC=PRIPLL 
#pragma config FPLLIDIV=DIV_2, FPLLMUL=MUL_20, FPLLODIV=DIV_1
#pragma config FPBDIV=DIV_1, FWDTEN=OFF, CP=OFF, BWP=OFF


#define SYS_FREQ (80000000L)
#define ASSERT(x) while(!x);

#include <plib.h>
#include <stdlib.h>
//#include <sys/appio.h>

#include "main_services.h"
#include "RTLL/app_services.h"

//Declare the prototypes of the library function
int (*add)(int  , int );
void (*mul)(int * , int *, int * );


/*******************************************************************
    Demo to show how to call "myLib" functions from the application
    using RTLL technique.
*********************************************************************/
int main( void)
{
    int addFirst = 10;
    int addSecond = 20;
    int addResult = 0;
    int mulFirst = 15;
    int mulSecond = 5;
    int mulResult = 0;
    int *ptr;
    UINT pbClk;
    void* hMyLib; // My Library handle
 
    // Setup configuration
    pbClk = SYSTEMConfig(SYS_FREQ, SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE);
		
    // get the library handle using dlopen() function.
    // TODO: Input library name and library address to dlopen()
    hMyLib = dlopen( "@MyMathLibrary" , (void*)0x9D06E000 ); // Pass Library name and the absolute address
    
    // Make sure handle is not NULL.
    ASSERT(hMyLib != 0);

    // get the handle to add function using dlsym().
    //TODO: Input the string name of the add function.
    add = dlsym(hMyLib,   "myAdd" ); // Pass Library handle and the Function name.

    // Make sure handle is not NULL.
    ASSERT(add != 0);

    // get the handle to add function using dlsym().
    //TODO: Input the string name of the multiplication function.
    mul = dlsym(hMyLib,   "myMul" ); // Pass Library handle and the Function name.
    ASSERT(mul != 0);  

    
    // Call the add function of the library.
    addResult = add(addFirst, addSecond);
    
    // Verify the result of the "add" operation.
    if(addResult != (addFirst + addSecond))
    {
        // Error!
        while(1);
    }
    
    // Call the mul function of the library.
    mul(&mulFirst, &mulSecond, &mulResult);

    // Verify the result of the "mul" operation.
    if(mulResult != (mulFirst * mulSecond))
    {
        //Error!
        while(1);
    }
	
	
    // Hurray! successful.
    while(1);
 
    return 0;
} // main


	